####-------- INTERSECTION OF GFF FILE AND BED FILE --------####
# Autor: Fadwa EL KHADDAR
# Lab : DIADE - IRD
# University : Montpellier - France

import csv
import os
import subprocess
import numpy as np
import pandas as pd
import  glob
import sys
import matplotlib
import matplotlib.pyplot as plt
import argparse
import re
from tkinter import *
matplotlib.use('TkAgg')
print("---- THIS SCRIPT AIMS TO APPLY INTERSECTION OF GFF FILE AND BED FILE ----")
print()

#   Define Arguments
parser = argparse.ArgumentParser(description="Venn Diagramm of gff and bed files")
parser.add_argument('-gff', type=str, help="Path to gff file")
parser.add_argument('-bed', type=str, help="Path to bed file")
parser.add_argument('-feature', type=str, default= all, help="Chosen feature : CDS, gene, exon")
parser.add_argument('-p', type=str,  required=False,default=None, help="Write TRUE if you want identify methylation on -1 and +1 gene")
parser.add_argument('-n', type=int, required=False, default=100, help="Number of bases to count before and after gene" )
parser.add_argument('-o', type=str, help="Path to output folder")

args = parser.parse_args()

gff = args.gff
bed = args.bed
output = args.o
feature= args.feature

#   Function when -p and -n are defined by the user
def add_promoter_terminator(start, end):
    promoter_end = start - 1
    terminator_start = end + 1

    promoter_line = f"{chromosome}\t{start-args.n}\t{promoter_end}\tpromotor"
    terminator_line = f"{chromosome}\t{terminator_start}\t{end+args.n}\tterminator"
    return promoter_line, terminator_line

#   Extract filenames of gff and bed files
filebasename = os.path.splitext(os.path.basename(bed))[0]
filebasename1= os.path.splitext(os.path.basename(gff))[0]
print(f"You are working on these files : \n - Gff file : {filebasename1} \n - bed file : {filebasename}")

#### FILTERS :
if feature != "all":
    # Extract only column 1,4,5,3 from gff and filter only lines of the chosen feature
    command1 = subprocess.Popen(
        f'awk -v OFS="\t" \'$3 != "mRNA" {{print $1 , $4 , $5 , $3}}\' {gff} | grep -w {feature}', shell=True,
        stdout=subprocess.PIPE)
    output1, errors1 = command1.communicate()
    count_features = output1.decode('utf-8').strip().split('\n')
    nb_features = len(count_features)

    # Bedtools Intersect of bed file constructed from gff file and bed file
    command2 = subprocess.Popen(f"bedtools intersect -wao -a stdin -b {bed}", shell=True, stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE)
    output2, errors2 = command2.communicate(input=output1)
    intersection = output2.decode('utf-8').strip()
    # Preparing columns to construct dataframe
    columns = ["chromosome_f1", "start_f1", "end_f1", "feature", "chromosome_f2", "start_f2", "end_f2", "methylation"]
    # Preparing rows, this line devise une str intersection on list of list, where each sub-list represent a line of intersection table
    rows = [line.split('\t') for line in intersection.split('\n') if line]
    # build dataframe
    dataframe = pd.DataFrame(rows, columns=columns)
    dataframe = dataframe[dataframe['methylation'].astype(int) != 0] # drop lines containing 0 in methylation
    # store the dataframe on csv file
    dataframe.to_csv(f"{output}/{feature}_Intersection_gff_vs_{filebasename}.csv")

    if feature == "gene":
        command1 = subprocess.Popen(
            f'awk -v OFS="\t" \'$3 != "mRNA" {{print $1 , $4 , $5 , $3}}\' {gff} | grep -w gene', shell=True,
            stdout=subprocess.PIPE)
        output1, errors1 = command1.communicate()
        count_features = output1.decode('utf-8').strip().split('\n')
        # This part of programm aims to add rows of promotor and terminator.
        if args.p is not None and args.n is not None:
            extended_results = []
            for result in count_features:
                chromosome, start, end, feature_type = result.split('\t')
                promoter_line, terminator_line = add_promoter_terminator(int(start), int(end))
                extended_results.append(promoter_line)
                extended_results.append(result)
                extended_results.append(terminator_line)
            output_file = f'{output}/results.csv' # this file will be deleted after this execution
            with open(output_file, 'w', newline='') as csvfile:
                writer = csv.writer(csvfile, delimiter='\t')
                for result in extended_results:
                    writer.writerow(result.split('\t'))
            csvfile.close()
            # the csv file "results.csv" will be used in the intersection
            command2 = subprocess.Popen(f"bedtools intersect -wao -a {output_file} -b {bed}", shell=True,stdout=subprocess.PIPE)
            output2, errors2 = command2.communicate()
            intersection = output2.decode('utf-8').strip()
            os.remove(output_file) #delete the file
            # Preparing columns to construct dataframe
            columns = ["chromosome_f1", "start_f1", "end_f1", "feature", "chromosome_f2", "start_f2", "end_f2", "methylation"]
            # Preparing rows, this line devise une str intersection on list of list, where each sub-list represent a line of intersection table
            rows= [line.split('\t') for line in intersection.split('\n') if line]
            dataframe = pd.DataFrame(rows, columns=columns)
            dataframe = dataframe[dataframe['methylation'].astype(int) != 0]
            # calculate the lenght of genes and add as a column.
            # calculate the 'position_methylation' which is the position of the methylation inside the gene: the gene start from 1 not 0
            dataframe['gene_length'] = (dataframe['end_f1'].astype(int) - dataframe['start_f1'].astype(int)) - 1
            dataframe['position_methylation'] = dataframe['start_f2'].astype(int) - dataframe['start_f1'].astype(int)
            # Calculate each tiers of gene and build categories of first tiers, second and third tiers.
            third = dataframe['gene_length'].astype(int) // 3.0
            dataframe['gene_tiers'] = np.select(
                [dataframe['position_methylation'] < third,
                 dataframe['position_methylation'] < third * 2,
                 dataframe['position_methylation'] >= third * 2],
                ['first', 'second', 'third'],
                default='unknown')

            # these last lines of programm add for all "gene , terminator, promotor" the gene tiers,
            # and the lines below rename the value of gene_tiers of promotor and terminator
            dataframe.loc[dataframe["feature"].astype(str) == "promotor", "gene_tiers"] = "promotor"
            dataframe.loc[dataframe["feature"].astype(str) == "terminator", "gene_tiers"] = "terminator"

            # Definition of categories
            cat_order = ['promotor', 'first', 'second', 'third', 'terminator']
            dataframe['gene_tiers'] = pd.Categorical(dataframe['gene_tiers'], categories=cat_order, ordered=True)
            methylated_counts = dataframe.groupby('gene_tiers').size().reindex(cat_order)
            # Plotting
            plt.bar(methylated_counts.index, methylated_counts.values)
            plt.title(f'Methylation Counts by Gene Tiers using {args.n} bases as promotor and terminator')
            plt.xlabel('Gene Tiers')
            plt.ylabel('Number of methylated cytosine')
            plt.show()
        else:
            command2 = subprocess.Popen(f"bedtools intersect -wao -a stdin -b {bed}", shell=True, stdin=subprocess.PIPE,
                                        stdout=subprocess.PIPE)
            output2, errors2 = command2.communicate(input=output1)
            intersection = output2.decode('utf-8').strip()
            columns = ["chromosome_f1", "start_f1", "end_f1", "feature", "chromosome_f2", "start_f2", "end_f2","methylation"]
            rows = [line.split('\t') for line in intersection.split('\n') if line]
            dataframe = pd.DataFrame(rows, columns=columns)
            methylated_counts = dataframe.groupby(['chromosome_f1', 'start_f1', 'end_f1']).size().reset_index(name='Methylated_cytosine')
            dataframe = dataframe[dataframe['methylation'].astype(int) != 0]
            methylated_counts.to_csv(f"{output}/{feature}_Intersection_gff_vs_{filebasename}.csv")
else:
    command1 = subprocess.Popen(f'awk -v OFS="\t" \'$3 != "mRNA" {{print $1 , $4 , $5 , $3}}\' {gff} ', shell=True, stdout=subprocess.PIPE)  #stdout=subprocess.PIPE, qui permet de récupérer la sortie de la commande
    command2 = subprocess.Popen(f"bedtools intersect -wao -a stdin -b {bed}", shell=True, stdin=command1.stdout, stdout=subprocess.PIPE) #stdin est utilisée pour lire les résultats de la première commande, et la sortie est également redirigée vers un tube, en utilisant subprocess.PIPE
    output3, errors = command2.communicate()
    intersection = output3.decode('utf-8').strip()
    columns = ["chromosome_f1", "start_f1", "end_f1", "feature", "chromosome_f2", "start_f2", "end_f2","methylation"]
    rows = [line.strip().split('\t') for line in intersection.split('\n') if line]
    dataframe = pd.DataFrame(rows, columns=columns)
    dataframe = dataframe[dataframe['methylation'].astype(int) != 0]
    dataframe.to_csv(f"{output}/{feature}_Intersection_gff_vs_{filebasename}.csv")

"""
#### Histogramme de nombre de méthylation pour chaque chromosome :

df= pd.read_csv(f"{output}/{feature}_Intersection_gff_vs_{filebasename}.csv")
df['chromosome_f1']=np.where(df['chromosome_f1']=='NC_003070.9', 'chr1', df['chromosome_f1'])
df['chromosome_f1']=np.where(df['chromosome_f1']=='NC_003071.7', 'chr2', df['chromosome_f1'])
df['chromosome_f1']=np.where(df['chromosome_f1']=='NC_003074.8', 'chr3', df['chromosome_f1'])
df['chromosome_f1']=np.where(df['chromosome_f1']=='NC_003075.7', 'chr4', df['chromosome_f1'])
df['chromosome_f1']=np.where(df['chromosome_f1']=='NC_003076.8', 'chr5', df['chromosome_f1'])
df['chromosome_f1']=np.where(df['chromosome_f1']=='NC_037304.1', 'MT', df['chromosome_f1'])
df['chromosome_f1']=np.where(df['chromosome_f1']=='NC_000932.1', 'Pltd', df['chromosome_f1'])

methylation_by_chromosome = df.groupby('chromosome_f1')['Methylated_cytosine'].sum()
plt.bar(methylation_by_chromosome.index, methylation_by_chromosome.values, color='green')
plt.xlabel('Chromosome')
plt.ylabel('Nombre de cytosines méthylées')
plt.title(f"Distribution des méthylations dans les {feature} groupé en fonction les chromosomes")
plt.show()

"""
