#!/bin/bash
​
### BY EL KHADDAR FADWA 
​
#SBATCH -J DeepSignal_Plant

#SBATCH --mail-user fadwael.khaddar@ird.fr
#SBATCH --mail-type=ALL
​
#SBATCH -e DeepSignal_Plant."%j".err
#SBATCH -o DeepSignal_Plant."%j".out
​
#SBATCH -p gpu
#SBATCH -A gpu_group
#SBATCH -c 8
#SBATCH --gres=gpu:4


# Loading module


module load system/Miniconda2/4.3.31
module load system/cuda/11.4
module load bioinfo/tombo/1.5.1
module load bioinfo/guppy-gpu/6.3.7

# Arguments 

yml_file=$1
INPUT=$2
OUTPUT=$3
NAME=$4
REF=$5
MODEL=$6

# RUNNING GUPPY 

echo " BASECALLING USING GUPPY "

# Paramètre d'utilisation guppy 

guppy_conf="dna_r9.4.1_450bps_hac_prom.cfg"



#	1. Basecalling

guppy_basecaller -i ${INPUT} -r -s ${OUTPUT} \
  --config $guppy_conf \
  --device CUDA:0

# 	2. Concatenation of fastq

cd ${OUTPUT}/pass/ 

cat *.fastq > ${NAME}_Guppy.fastq

# RUNNING TOMBO

# Paramètre d'utilisation TOMBO

basecall_group="Basecall_1D_000"
basecall_subgroupe="BaseCalled_template"
corrected_group="RawGenomeCorrected_000"


echo " RUNNING TOMBO "

cd 

cd ${INPUT} 

wget https://github.com/nanoporetech/vbz_compression/releases/download/v1.0.1/ont-vbz-hdf-plugin-1.0.1-Linux-x86_64.tar.gz

tar zxvf ont-vbz-hdf-plugin-1.0.1-Linux-x86_64.tar.gz

export HDF5_PLUGIN_PATH=ont-vbz-hdf-plugin-1.0.1-Linux/usr/local/hdf5/lib/plugin/

#	1. Tombo preprocess 


tombo preprocess annotate_raw_with_fastqs --fast5-basedir ${INPUT}   --fastq-filenames ${OUTPUT}/pass/${NAME}_Guppy.fastq  --sequencing-summary-filenames ${OUTPUT}/sequencing_summary.txt   --basecall-group $basecall_group --basecall-subgroup $basecall_subgroupe --overwrite --processes 10



#	2. Tombo resquiggle 


tombo resquiggle ${INPUT} ${REF}  --processes 10 --corrected-group $corrected_group   --basecall-group $basecall_group --overwrite

### RUNNING DEEPSIGNAL_PLANT 

echo " RUNNING DEEPSIGNAL-PLANT " 

# Conda environement creation

conda env create -f $yml_file 

source activate deepsignalpenv


# Paramètre d'utilisation de DeepSignal_Plant 

nproc=20
ngpu=4

#	1. Call modifications

CUDA_VISIBLE_DEVICES=0 deepsignal_plant call_mods --input_path ${INPUT}  --model_path ${MODEL}   --result_file  ${OUTPUT}/${NAME}_call_mods.tsv --corrected_group $corrected_group --motifs C --nproc $nproc --nproc_gpu $ngpu


#	2. Call mod frequency: tsv 

deepsignal_plant call_freq --input_path ${OUTPUT}/${NAME}_call_mods.tsv --result_file ${OUTPUT}/${NAME}_call_mods.frequency.tsv


# 	3. Call mod frenquency: bed

deepsignal_plant call_freq --input_path ${OUTPUT}/${NAME}_call_mods.tsv --result_file ${OUTPUT}/${NAME}_call_mods.frequency.bed --bed --sort 









​
